@@ -78,7 +78,7 @@ namespace a {
   typedef int f2_type(int, int);
 
   void test_f2() {
-    ::f2_type(1, 2); // expected-error {{function-style cast to a builtin type can only take one argument}}
+    ::f2_type(1, 2); // expected-error {{excess elements in scalar initializer}}
   }
 }
 
