@@ -14,7 +14,7 @@ void test2(int a) {
 }
 
 // PR5286
-void test3(int a) {  // expected-warning {{function could be attribute 'noreturn'}}
+void test3(int a) {  // expected-warning {{function 'test3' could be declared with attribute 'noreturn'}}
   while (1) {
     if (a)
       @throw (id)0;
@@ -39,3 +39,12 @@ NSString *rdar_4289832() {  // no-warning
     }
 }
 
+void exit(int) __attribute__((noreturn));
+@interface rdar10098695
+@end
+
+@implementation rdar10098695
+- (void)method { // expected-warning{{method 'method' could be declared with attribute 'noreturn'}}
+  exit(1);
+}
+@end
