@@ -50,3 +50,18 @@ namespace local_class_with_virtual_functions {
   struct S { };
   void test() { f<S>(); }
 }
+
+namespace PR8801 {
+  template<typename T>
+  void foo() {
+    class X;
+    typedef int (X::*pmf_type)();
+    class X : public T { };
+    
+    pmf_type pmf = &T::foo;
+  }
+
+  struct Y { int foo(); };
+
+  template void foo<Y>();
+}
